#ifndef BRICK_BASE_H
#define BRICK_BASE_H

#include "pievaluator.h"
#include "piconfig.h"
#include "pivariable.h"

inline double residue(double f, double s) {return f - floor(f / s) * s;}
inline double sign(double f) {return (f == 0.) ? 0. : ((f > 0.) ? 1. : -1.);}
inline double dbool(double d) {return d > 0 ? 1. : 0;}
inline double randomd() {return (double)random() / RAND_MAX * 2. - 1.;}
double randomn(double dv = 0., double sv = 1.);

class BrickComposite;
class BrickManager;

class BrickBase
{
	friend class BrickManager;
	friend void buildTree(PIVector<BrickBase * > & bricks, PIVector<PIVector<BrickBase * > > & tree);
	friend PIVector<PIVector<BrickBase * > > buildSubtree(PIVector<BrickBase * > & bricks, PIVector<PIVector<BrickBase * > > & tree);
public:
	BrickBase(int inputs_num = 1, int outputs_num = 1, int parameters_num = 0);
	virtual ~BrickBase() {;}
	virtual BrickBase * copy() {return new BrickBase(*this);}
	
	enum IOType {
		Fixed = 0x0,
		VariableInputs = 0x01,
		VariableOutputs = 0x02,
		VariableInOuts = VariableInputs | VariableOutputs
	};
	enum Mode {Synchronous, Asynchronous};
	enum ParameterType {Bool, Integer, Float, String, File};
	
	struct Parameter {
		Parameter() {type = BrickBase::Float;}
		Parameter(const char * v) {setValue(v);}
		Parameter(const bool & v) {setValue(v);}
		Parameter(const int & v) {setValue(v);}
		Parameter(const double & v) {setValue(v);}
		Parameter(const PIString & v) {setValue(v);}
		void setType(const ParameterType & t) {type = t;}
		void setValue(const char * v, bool withType = true) {setValue(PIString(v), withType);}
		void setValue(const bool & v, bool withType = true) {if (withType) type = BrickBase::Bool; value = v;}
		void setValue(const int & v, bool withType = true) {if (withType) type = BrickBase::Integer; value = v;}
		void setValue(const double & v, bool withType = true) {if (withType) type = BrickBase::Float; value = v;}
		void setValue(const PIString & v, bool withType = true) {if (withType) type = BrickBase::String; value = v;}
		void setValue(const PIVariant & v, bool withType = true);
		void setValueOnly(const PIString & v) {value.setValueOnly(v);}
		bool toBool() const {return value.vBool;}
		int toInt() const {return value.vInt;}
		uint toUInt() const {return value.vInt;}
		double toFloat() const {return value.vDouble;}
		PIString toString() const {return value.vString;}
		ParameterType type;
		PIVariant value;
	};
	
	virtual void reset_specified() {;}
	virtual void inCountChanged(int count) {;}
	virtual void outCountChanged(int count) {;}
	virtual void started() {;}
	virtual void finished() {;}
	virtual void parameterChanged(int index) {;}
	virtual void saveInputsToDefault() {defInputs = inputs;}
	virtual void resetOutputs() {outputs.assign(outputs.size(), 0.);}
	virtual void reset() {finished(); inputs = defInputs; dt = pt = time_ = 0.; reset_specified(); resetOutputs(); started();}
	virtual void proceedConnections(bool compositeToo = false);
	bool tick(double time);
	inline int frequencyDivider() const {return freqDivider_;}
	inline void setFrequencyDivider(const int divider) {freqDivider_ = divider;}
	inline int inCount() const {return inputsCount;}
	inline int outCount() const {return outputsCount;}
	inline int paramCount() const {return parametersCount;}
	inline double input(const int index = 0) const {return inputs[index];}
	inline double output(const int index = 0) const {return outputs[index];}
	inline const PIVariant & parameter(const int index = 0) const {return parameters[index].value;}
	inline const BrickBase::ParameterType & parameterType(const int index = 0) const {return parameters[index].type;}
	inline double * input_ptr(const int index = 0) {return &inputs[index];}
	inline double * output_ptr(const int index = 0) {return &outputs[index];}
	inline double * parameter_ptr(const int index = 0) {return &(parameters[index].value.vDouble);}
	inline void setInputValue(const int index, double value) {inputs[index] = value;}
	inline void setOutputValue(const int index, double value) {outputs[index] = value;}
	inline void setDefaultInputValue(const int index, double value) {defInputs[index] = value;}
	inline void setParameterValue(const int index, const char * value, bool withType = false) {setParameterValue(index, PIString(value), withType);}
	inline void setParameterValue(const int index, const bool & value, bool withType = false) {parameters[index].setValue(value, withType); parameterChanged(index);}
	inline void setParameterValue(const int index, const int & value, bool withType = false) {parameters[index].setValue(value, withType); parameterChanged(index);}
	inline void setParameterValue(const int index, const double & value, bool withType = false) {parameters[index].setValue(value, withType); parameterChanged(index);}
	inline void setParameterValue(const int index, const PIString & value, bool withType = false) {parameters[index].setValue(value, withType); parameterChanged(index);}
	inline void setParameterValue(const int index, const PIVariant & value, bool withType = false) {parameters[index].setValue(value, withType); parameterChanged(index);}
	inline void setParameterValue(const int index, const BrickBase::Parameter & value) {parameters[index] = value; parameterChanged(index);}
	inline void setParameterValueOnly(const int index, const PIString & value) {parameters[index].setValueOnly(value); parameterChanged(index);}
	inline void setName(const PIString & name) {name_ = name;}
	inline void setInName(const int index, const PIString & name) {inNames[index] = name;}
	inline void setOutName(const int index, const PIString & name) {outNames[index] = name;}
	inline void setParamName(const int index, const PIString & name) {paramNames[index] = name;}
	inline void setInputsCount(const int count) {inputs.resize(count); inputsCount = inputs.size(); inNames.resize(count); inCountChanged(count);}
	inline void setOutputsCount(const int count) {outputs.resize(count); outputsCount = outputs.size(); outNames.resize(count); outCountChanged(count);}
	inline void setParametersCount(const int count, const char * val) {parameters.resize(count, BrickBase::Parameter(PIString(val))); paramNames.resize(count); parametersCount = parameters.size();}
	inline void setParametersCount(const int count, const bool & val) {parameters.resize(count, BrickBase::Parameter(val)); paramNames.resize(count); parametersCount = parameters.size();}
	inline void setParametersCount(const int count, const int val) {parameters.resize(count, BrickBase::Parameter(val)); paramNames.resize(count); parametersCount = parameters.size();}
	inline void setParametersCount(const int count, const double & val = 0.) {parameters.resize(count, BrickBase::Parameter(val)); paramNames.resize(count); parametersCount = parameters.size();}
	inline void setParametersCount(const int count, const PIString & val) {parameters.resize(count, BrickBase::Parameter(val)); paramNames.resize(count); parametersCount = parameters.size();}
	inline const PIVector<BrickBase::Parameter> & getParameters() const {return parameters;}
	inline void setInOutCount(const int count) {setInputsCount(count); setOutputsCount(count);}
	inline int ioType() const {return io_Type;}
	inline PIString name() const {return name_;}
	inline PIString libName() const {return lib;}
	inline PIString typeName() const {return type;}
	inline PIString codeName() const {return codeName_;}
	inline PIString parametersName() const {return parametersName_;}
	inline PIString inName(const int index) const {return inNames[index];}
	inline PIString outName(const int index) const {return outNames[index];}
	inline PIString paramName(const int index) const {return paramNames[index];}
	inline PIString note() const {return note_;}
	inline void setNote(const PIString & str) {note_ = str;}
	inline PIString tag() const {return tag_;}
	inline void setTag(const PIString & str) {tag_ = str;}
	inline BrickManager * manager() const {return manager_;}
	inline void setManager(BrickManager * m) {manager_ = m;}
	inline bool isInitial() const {return initial;}
	inline void setInitial(const bool yes = true) {initial = yes;}
	inline bool isComposite() const {return composite;}
	inline void setComposite(const bool yes = true) {composite = yes;}
	inline bool isBuffered() const {return buffered;}
	inline void setBuffered(const bool yes = true) {buffered = yes;}
	inline bool isInteractive() const {return interactive;}
	inline void setInteractive(const bool yes = true) {interactive = yes;}
	inline bool isRealTimeOnly() const {return rtOnly;}
	inline void setRealTimeOnly(const bool yes = true) {rtOnly = yes;}
	inline BrickComposite * toComposite() {return reinterpret_cast<BrickComposite * >(this);}
	
	struct Connection {
		int out_num;
		BrickBase * brick_to;
		int in_num;
		Connection() {;}
		Connection(int out_n, BrickBase * b_to, int in_n) {brick_to = b_to; out_num = out_n; in_num = in_n;}
	};
	
	inline void addConnection(Connection conn) {connections.push_back(conn);}
	inline void clearConnections() {connections.clear();}
	inline BrickBase::Connection connection(int index) const {return connections[index];}
	inline int connectionsCount() const {return connections.size();}
	inline void removeConnection(int index) {connections.erase(connections.begin() + index);}
	
	bool lastTick, userBool;
	double time_, userDouble;
	int level_, userInt;
	PIVector<double> buffer;

	static bool connect(BrickBase * b_from, int out_num, BrickBase * b_to, int in_num);
	static bool connect(BrickBase * b_from, int out_num, BrickBase & b_to, int in_num) {return connect(b_from, out_num, &b_to, in_num);}
	static bool connect(BrickBase & b_from, int out_num, BrickBase * b_to, int in_num) {return connect(&b_from, out_num, b_to, in_num);}
	static bool connect(BrickBase & b_from, int out_num, BrickBase & b_to, int in_num) {return connect(&b_from, out_num, &b_to, in_num);}

	static bool disconnect(BrickBase * b_from, int out_num, BrickBase * b_to, int in_num);
	static bool disconnect(BrickBase * b_from, int out_num, BrickBase & b_to, int in_num) {return disconnect(b_from, out_num, &b_to, in_num);}
	static bool disconnect(BrickBase & b_from, int out_num, BrickBase * b_to, int in_num) {return disconnect(&b_from, out_num, b_to, in_num);}
	static bool disconnect(BrickBase & b_from, int out_num, BrickBase & b_to, int in_num) {return disconnect(&b_from, out_num, &b_to, in_num);}
	
protected:
	virtual inline bool tick_body(double time) {return true;}
	
	PIVector<double> inputs, outputs, defInputs;
	PIVector<Connection> connections;
	PIVector<Parameter> parameters;
	PIVector<PIString> inNames, outNames, paramNames;
	PIString name_, parametersName_, lib, type, codeName_, note_, tag_;
	int inputsCount, outputsCount, parametersCount, io_Type, tmp_, freqDivider_, freqCurrent_;
	double dt, pt;
	bool initial, ticked, composite, buffered, interactive, rtOnly;
	BrickManager * manager_;
};


#endif // BRICK_BASE_H
