#ifndef BRICK_EMITS_H
#define BRICK_EMITS_H

#include "brick_base.h"
#include "brick_manager.h"


class BrickEmitBase: public BrickBase {
public: BrickEmitBase(int inputs_num = 1, int outputs_num = 1, int parameters_num = 0): BrickBase(inputs_num, outputs_num, parameters_num) {lib = "Emitters";}
};


class BrickEmitConst: public BrickEmitBase {
public:
	BrickEmitConst(double value = 1.): BrickEmitBase(0, 1, 1) {codeName_ = "BrickEmitConst"; parameters[0].setValue(value); type = name_ = "Const"; paramNames[0] = "Const";}
	virtual BrickBase * copy() {return new BrickEmitConst(*this);}
	void setValue(double value) {parameters[0].setValue(value);}
	virtual bool tick_body(double time) {outputs[0] = parameters[0].toFloat(); return true;}
};


class BrickEmitLinear: public BrickEmitBase {
public:
	BrickEmitLinear(double start_value = 0., double slope = 1.): BrickEmitBase(2, 1) {codeName_ = "BrickEmitLinear"; setParameters(start_value, slope); type = name_ = "Linear"; inNames[0] = "Start"; inNames[1] = "Slope";}
	virtual BrickBase * copy() {return new BrickEmitLinear(*this);}
	enum Inputs {StartValue, Slope};
	void setParameters(double start_value, double slope) {inputs[StartValue] = start_value; inputs[Slope] = slope;}
	virtual bool tick_body(double time);
};


class BrickEmitSin: public BrickEmitBase {
public:
	BrickEmitSin(double amplitude = 1., double frequency = 1., double phase_shift = 0.): BrickEmitBase(3, 1) {codeName_ = "BrickEmitSin"; setParameters(amplitude, frequency, phase_shift); type = name_ = "Sin"; inNames[0] = "Amplitude"; inNames[1] = "Frequency"; inNames[2] = "Phase Shift";}
	virtual BrickBase * copy() {return new BrickEmitSin(*this);}
	enum Inputs {Amplitude, Frequency, PhaseShift};
	void setParameters(double amplitude, double frequency, double phase_shift) {inputs[Amplitude] = amplitude; inputs[Frequency] = frequency; inputs[PhaseShift] = phase_shift;}
	virtual bool tick_body(double time);
};


class BrickEmitDelta: public BrickEmitBase {
public:
	BrickEmitDelta(double time = 0.): BrickEmitBase(1, 1) {codeName_ = "BrickEmitDelta"; setTime(time); type = name_ = "Delta"; inNames[0] = "Time";}
	virtual BrickBase * copy() {return new BrickEmitDelta(*this);}
	enum Inputs {Time};
	void setTime(double time) {inputs[Time] = time;}
	virtual bool tick_body(double time);
};


class BrickEmitStep: public BrickEmitBase {
public:
	BrickEmitStep(double time = 0., double start = 0., double finish = 1.): BrickEmitBase(3, 1) {codeName_ = "BrickEmitStep"; setParameters(time, start, finish); type = name_ = "Step"; inNames[0] = "Time"; inNames[1] = "Start Value"; inNames[2] = "Finish Value";}
	virtual BrickBase * copy() {return new BrickEmitStep(*this);}
	enum Inputs {Time, StartValue, FinishValue};
	void setParameters(double time, double start, double finish) {inputs[Time] = time; inputs[StartValue] = start; inputs[FinishValue] = finish;}
	virtual bool tick_body(double time);
};


class BrickEmitPulse: public BrickEmitBase {
public:
	BrickEmitPulse(double ampl = 1., double freq = 1., double ratio = 0.5, double shift = 0.): BrickEmitBase(4, 1) {codeName_ = "BrickEmitPulse"; setParameters(ampl, freq, ratio, shift); type = name_ = "Pulse"; inNames[0] = "Amplitude"; inNames[1] = "Frequency"; inNames[2] = "Ratio"; inNames[3] = "Phase Shift";}
	virtual BrickBase * copy() {return new BrickEmitPulse(*this);}
	enum Inputs {Amplitude, Frequency, Ratio, PhaseShift};
	void setParameters(double ampl, double freq, double ratio, double shift) {inputs[Amplitude] = ampl; inputs[Frequency] = freq; inputs[Ratio] = ratio; inputs[PhaseShift] = shift;}
	virtual bool tick_body(double time);
};


class BrickEmitStrobe: public BrickEmitBase {
public:
	BrickEmitStrobe(): BrickEmitBase(0, 1) {codeName_ = "BrickEmitStrobe"; type = name_ = "Strobe";}
	virtual BrickBase * copy() {return new BrickEmitStrobe(*this);}
	virtual bool tick_body(double time) {outputs[0] = 1. - outputs[0]; return true;}
};


class BrickEmitTime: public BrickEmitBase {
public:
	BrickEmitTime(): BrickEmitBase(0, 1) {codeName_ = "BrickEmitTime"; type = name_ = "Time";}
	virtual BrickBase * copy() {return new BrickEmitTime(*this);}
	virtual bool tick_body(double time) {outputs[0] = time; return true;}
};


class BrickEmitFrequency: public BrickEmitBase {
public:
	BrickEmitFrequency(): BrickEmitBase(0, 1) {codeName_ = "BrickEmitFrequency"; type = name_ = "Frequency";}
	virtual BrickBase * copy() {return new BrickEmitFrequency(*this);}
	virtual bool tick_body(double time) {outputs[0] = manager_->freq; return true;}
};


class BrickEmitNoiseRandom: public BrickEmitBase {
public:
	BrickEmitNoiseRandom(double shift = 0., double scale = 1.): BrickEmitBase(2, 1) {codeName_ = "BrickEmitNoiseRandom"; setParameters(shift, scale); type = name_ = "Random"; inNames[0] = "Shift"; inNames[1] = "Scale";}
	virtual BrickBase * copy() {return new BrickEmitNoiseRandom(*this);}
	enum Inputs {Shift, Scale};
	void setParameters(double shift, double scale) {inputs[Shift] = shift; inputs[Scale] = scale;}
	virtual bool tick_body(double time) {outputs[0] = randomd() * inputs[Scale] + inputs[Shift]; return true;}
};


class BrickEmitNoiseNormal: public BrickEmitBase {
public:
	BrickEmitNoiseNormal(double shift = 0., double scale = 1.): BrickEmitBase(2, 1) {codeName_ = "BrickEmitNoiseNormal"; setParameters(shift, scale); type = name_ = "NormalNoise"; inNames[0] = "Shift"; inNames[1] = "Scale";}
	virtual BrickBase * copy() {return new BrickEmitNoiseNormal(*this);}
	enum Inputs {Shift, Scale};
	void setParameters(double shift, double scale) {inputs[Shift] = shift; inputs[Scale] = scale;}
	virtual bool tick_body(double time) {outputs[0] = randomn(inputs[Shift], inputs[Scale]); return true;}
};

#endif // BRICK_EMITS_H
