#ifndef BRICK_MATH_H
#define BRICK_MATH_H

#include "brick_base.h"
#include "brick_manager.h"


class BrickMathBase: public BrickBase {
public: BrickMathBase(int inputs_num = 1, int outputs_num = 1, int parameters_num = 0): BrickBase(inputs_num, outputs_num, parameters_num) {lib = "Mathematic";}
};


class BrickMathAmplifier: public BrickMathBase {
public:
	BrickMathAmplifier(double gain = 1.): BrickMathBase(2, 1) {codeName_ = "BrickMathAmplifier"; setGain(gain); type = name_ = "Gain"; inNames[1] = "Gain";}
	virtual BrickBase * copy() {return new BrickMathAmplifier(*this);}
	enum Inputs {Input, Gain};
	void setGain(double gain) {inputs[Gain] = gain;}
	virtual bool tick_body(double time) {outputs[0] = inputs[Gain] * inputs[Input]; return true;}
};


class BrickMathSum: public BrickMathBase {
public:
	BrickMathSum(int inputs_num = 2): BrickMathBase(inputs_num, 1, inputs_num) {
		codeName_ = "BrickMathSum";
		parameters.assign(inputs_num, 1.);
		type = name_ = "Sum";
		parametersName_ = "Preamps";
		inNames[0] = "";
		io_Type = BrickBase::VariableInputs;
	}
	virtual BrickBase * copy() {return new BrickMathSum(*this);}
	virtual void parameterChanged(int index) {inNames[index] = parameters[index].toFloat() >= 0. ? "+" : "-";}
	virtual void inCountChanged(int count) {setParametersCount(count, 1.);}
	void setPreamp(int input, double preamp) {parameters[input].setValue(preamp);}
	virtual bool tick_body(double time);
};


class BrickMathSign: public BrickMathBase {
public:
	BrickMathSign(): BrickMathBase(1, 1) {codeName_ = "BrickMathSign"; type = name_ = "Sign";}
	virtual BrickBase * copy() {return new BrickMathSign(*this);}
	enum Inputs {Input};
	virtual bool tick_body(double time) {outputs[0] = sign(inputs[Input]); return true;}
};


class BrickMathAbsolute: public BrickMathBase {
public:
	BrickMathAbsolute(): BrickMathBase(1, 1) {codeName_ = "BrickMathAbsolute"; type = name_ = "Abs";}
	virtual BrickBase * copy() {return new BrickMathAbsolute(*this);}
	enum Inputs {Input};
	virtual bool tick_body(double time) {outputs[0] = fabs(inputs[Input]); return true;}
};


class BrickMathMultiply: public BrickMathBase {
public:
	BrickMathMultiply(int inputs_num = 2): BrickMathBase(inputs_num, 1) {codeName_ = "BrickMathMultiply"; type = name_ = "Multiply"; inNames[0] = ""; io_Type = BrickBase::VariableInputs;}
	virtual BrickBase * copy() {return new BrickMathMultiply(*this);}
	virtual bool tick_body(double time);
};


class BrickMathDivide: public BrickMathBase {
public:
	BrickMathDivide(): BrickMathBase(2, 1) {codeName_ = "BrickMathDivide"; type = name_ = "Divide"; inNames[0] = "Divident"; inNames[1] = "Divider"; inputs[1] = 2.;}
	virtual BrickBase * copy() {return new BrickMathDivide(*this);}
	enum Inputs {Input, Divider};
	virtual bool tick_body(double time) {if (inputs[Divider] == 0.) return false; outputs[0] = inputs[Input] / inputs[Divider]; return true;}
};


class BrickMathPower: public BrickMathBase {
public:
	BrickMathPower(double power = 2.): BrickMathBase(2, 1) {codeName_ = "BrickMathPower"; setPower(power); type = name_ = "Power"; inNames[1] = "Power";}
	virtual BrickBase * copy() {return new BrickMathPower(*this);}
	enum Inputs {Input, Power};
	void setPower(double power) {inputs[Power] = power;}
	virtual bool tick_body(double time) {outputs[0] = pow(inputs[Input], inputs[Power]); return true;}
private:
	double p;
};


class BrickMathIntegral: public BrickMathBase {
public:
	BrickMathIntegral(double initial = 0.);
	virtual BrickBase * copy() {return new BrickMathIntegral(*this);}
	enum Inputs {Input, Enable, Reset, Initial};
	virtual void started();
	virtual void reset_specified() {KF.fromTF(TF); KF.X[0] = inputs[Initial];}
	virtual void parameterChanged(int index) {KF.setMethod((Solver::Method)parameters[0].toInt());}
	virtual bool tick_body(double time);
protected:
	Solver KF;
	TransferFunction TF;
};


class BrickMathIntegralSaturated: public BrickMathIntegral {
public:
	BrickMathIntegralSaturated(double initial = 0.): BrickMathIntegral(initial) {codeName_ = "BrickMathIntegralSaturated"; type = name_ = "IntegralSaturated"; setInputsCount(6); inputs[Min] = -1.; inputs[Max] = 1.; inNames[4] = "Min"; inNames[5] = "Max";}
	virtual BrickBase * copy() {return new BrickMathIntegralSaturated(*this);}
	enum Inputs {Input, Enable, Reset, Initial, Min, Max};
	virtual bool tick_body(double time) {BrickMathIntegral::tick_body(time); if (KF.X[0] < inputs[Min]) {KF.X[0] = inputs[Min]; outputs[0] = KF.X[0];} if (KF.X[0] > inputs[Max]) {KF.X[0] = inputs[Max]; outputs[0] = KF.X[0];} return true;}
};


class BrickMathDerivate: public BrickMathBase {
public:
	BrickMathDerivate(): BrickMathBase(1, 1) {codeName_ = "BrickMathDerivate"; v = 0.; type = name_ = "Derivate";}
	virtual BrickBase * copy() {return new BrickMathDerivate(*this);}
	virtual bool tick_body(double time);
private:
	double v;
};


class BrickMathDeadZone: public BrickMathBase {
public:
	BrickMathDeadZone(double zone = 1.): BrickMathBase(2, 2) {codeName_ = "BrickMathDeadZone"; setZone(zone); type = name_ = "DeadZone"; inNames[1] = "Zone"; outNames[1] = "In Zone";}
	virtual BrickBase * copy() {return new BrickMathDeadZone(*this);}
	enum Inputs {Input, Zone};
	enum Outputs {Output, InZone};
	void setZone(double zone) {inputs[Zone] = zone;}
	virtual bool tick_body(double time);
};


class BrickMathSaturation: public BrickMathBase {
public:
	BrickMathSaturation(double min = -1., double max = 1.): BrickMathBase(3, 3) {codeName_ = "BrickMathSaturation"; setRange(min, max); type = name_ = "Saturation"; inNames[1] = "Min"; inNames[2] = "Max"; outNames[1] = "< min"; outNames[2] = "> max";}
	virtual BrickBase * copy() {return new BrickMathSaturation(*this);}
	enum Inputs {Input, Min, Max};
	enum Outputs {Output, InMin, InMax};
	void setRange(double min, double max) {inputs[Max] = max; inputs[Min] = min;}
	virtual bool tick_body(double time);
};


class BrickMathQuantize: public BrickMathBase {
public:
	BrickMathQuantize(double step = 1.): BrickMathBase(2, 1) {codeName_ = "BrickMathQuantize"; setStep(step); type = name_ = "Quantize"; inNames[1] = "Step";}
	virtual BrickBase * copy() {return new BrickMathQuantize(*this);}
	enum Inputs {Input, Step};
	void setStep(double step) {inputs[Step] = step;}
	virtual bool tick_body(double time) {outputs[0] = round(inputs[Input] / inputs[Step]) * inputs[Step]; return true;}
};


class BrickMathRelay: public BrickMathBase {
public:
	BrickMathRelay(double size = 1., double active = 1., double inactine = 0.): BrickMathBase(4, 1) {codeName_ = "BrickMathRelay"; setParameters(size, active, inactine); v = 0.; type = name_ = "Relay"; inNames[1] = "Size"; inNames[2] = "Active Value"; inNames[3] = "Inactive Value";}
	virtual BrickBase * copy() {return new BrickMathRelay(*this);}
	virtual void reset_specified() {v = 0.;}
	enum Inputs {Input, Size, ActiveValue, InactiveValue};
	void setParameters(double size, double active, double inactine) {inputs[Size] = size; inputs[ActiveValue] = active; inputs[InactiveValue] = inactine;}
	virtual bool tick_body(double time);
private:
	double v;
};


class BrickMathDelayTicks: public BrickMathBase {
public:
	BrickMathDelayTicks(uint ticks = 1): BrickMathBase(1, 1, 1) {codeName_ = "BrickMathDelayTicks"; setDelay(ticks); type = name_ = "DelayTicks"; paramNames[0] = "Ticks"; parameters[0].setValue(1);}
	virtual BrickBase * copy() {return new BrickMathDelayTicks(*this);}
	virtual void reset_specified() {v.assign(v.size(), 0.);}
	void setDelay(int ticks) {parameters[0].setValue(ticks); v.resize(ticks); v.assign(ticks, 0.);}
	virtual bool tick_body(double time);
private:
	deque<double> v;
};


class BrickMathDelaySeconds: public BrickMathBase {
public:
	BrickMathDelaySeconds(double secs = 1.): BrickMathBase(1, 1, 1) {codeName_ = "BrickMathDelaySeconds"; setDelay(secs); type = name_ = "DelaySeconds"; paramNames[0] = "Seconds"; parameters[0].setValue(1.);}
	virtual BrickBase * copy() {return new BrickMathDelaySeconds(*this);}
	virtual void reset_specified() {v.assign(v.size(), 0.);}
	void setDelay(double secs) {parameters[0].setValue(secs);}
	virtual bool tick_body(double time);
private:
	deque<double> v;
	uint t;
};


class BrickMathThreshold: public BrickMathBase {
public:
	BrickMathThreshold(double value = 0.5): BrickMathBase(2, 1) {codeName_ = "BrickMathThreshold"; setThreshold(value); type = name_ = "Threshold"; inNames[1] = "Threshold";}
	virtual BrickBase * copy() {return new BrickMathThreshold(*this);}
	enum Inputs {Input, Threshold};
	void setThreshold(double value) {inputs[Threshold] = value;}
	virtual bool tick_body(double time) {outputs[0] = (inputs[Input] >= inputs[Threshold] ? 1. : 0.); return true;}
};


class BrickMathFunction: public BrickMathBase {
public:
	BrickMathFunction();
	virtual BrickBase * copy() {return new BrickMathFunction(*this);}
	virtual void parameterChanged(int index);
	virtual bool tick_body(double time);
private:
	PIEvaluator eval;
	complexd res;
};


class BrickMathFFT: public BrickMathBase {
public:
	BrickMathFFT();
	virtual BrickBase * copy() {return new BrickMathFFT(*this);}
	virtual bool tick_body(double time);
	virtual void started() {t = 0;}
	virtual void reset_specified() {t = 0;}
private:
	PIVector<complexd> v, o;
	uint t;
};


class BrickMathCopy: public BrickMathBase {
public:
	BrickMathCopy(): BrickMathBase(1, 2) {codeName_ = "BrickMathCopy"; type = name_ = "Copy"; outNames[0] = "0"; io_Type = VariableOutputs;}
	virtual BrickBase * copy() {return new BrickMathCopy(*this);}
	virtual bool tick_body(double time) {for (int i = 0; i < outputsCount; ++i) outputs[i] = inputs[0]; return true;}
};


class BrickMathBessel: public BrickMathBase {
public:
	BrickMathBessel(): BrickMathBase(1, 1, 2) {codeName_ = "BrickMathBessel"; type = name_ = "Bessel"; paramNames[0] = "Kind"; paramNames[1] = "Order"; setParameterValue(0, 0); setParameterValue(1, 0);}
	virtual BrickBase * copy() {return new BrickMathBessel(*this);}
	virtual void parameterChanged(int index);
	virtual bool tick_body(double time);
private:
	int o, k;
};


#endif // BRICK_MATH_H
